package backend

import (
	"errors"
	"strings"

	"github.com/bwmarrin/discordgo"

	"TerraOceanBot/db"
	"TerraOceanBot/discord/config"
)

var ErrMemberNotFound = errors.New("discord/backend: the requested member was not found")

func GetAllMembers(s *discordgo.Session) ([]*discordgo.Member, error) {
	guild, err := s.State.Guild(config.GuildID)
	if err != nil {
		guild, err = s.Guild(config.GuildID)
	}
	if err != nil {
		return nil, err
	}
	return guild.Members, nil
}

func GetMemberFromUserFriendlyName(s *discordgo.Session, user string) (*discordgo.Member, error) {
	allMembers, err := GetAllMembers(s)
	if err != nil {
		return nil, err
	}
	if strings.HasPrefix(user, "@") {
		user = strings.TrimPrefix(user, "@")
	} else if strings.HasSuffix(user, ">") {
		if strings.HasPrefix(user, "<@!") {
			user = strings.TrimSuffix(strings.TrimPrefix(user, "<@!"), ">")
		} else if strings.HasPrefix(user, "<@") {
			user = strings.TrimSuffix(strings.TrimPrefix(user, "<@"), ">")
		}
	}
	if strings.ContainsRune(user, '#') {
		list := strings.Split(user, "#")
		username := strings.Join(list[:len(list)-1], "#")
		discriminator := list[len(list)-1]
		for _, v := range allMembers {
			if v.User.Username == username && v.User.Discriminator == discriminator {
				return v, nil
			}
		}
	}
	for _, v := range allMembers {
		if v.User.ID == user {
			return v, nil
		}
	}
	for _, v := range allMembers {
		if v.Nick == user {
			return v, nil
		}
	}
	for _, v := range allMembers {
		if v.User.Username == user {
			return v, nil
		}
	}
	for _, v := range allMembers {
		mcUsername, err := db.GetMinecraftUsername(v.User.ID)
		if err == db.ErrNotFound {
			continue
		}
		if err != nil {
			return nil, err
		}
		if mcUsername == user {
			return v, nil
		}
	}
	return nil, ErrMemberNotFound
}
